# -*- coding: utf-8 -*-
"""
orm sql operators package.
"""

from pyrin.packaging.base import Package


class ORMSQLOperatorsPackage(Package):
    """
    orm sql operators package class.
    """

    NAME = __name__
