# -*- coding: utf-8 -*-
"""
utilities string package.
"""

from pyrin.packaging.base import Package


class UtilitiesStringPackage(Package):
    """
    utilities string package class.
    """

    NAME = __name__
