# -*- coding: utf-8 -*-
"""
converters package.
"""

from pyrin.packaging.base import Package


class ConvertersPackage(Package):
    """
    converters package class.
    """

    NAME = __name__
    DEPENDS = []
