# -*- coding: utf-8 -*-
"""
deserializer handlers package.
"""

from pyrin.packaging.base import Package


class DeserializerHandlersPackage(Package):
    """
    deserializer handlers package class.
    """

    NAME = __name__
    DEPENDS = []
