# -*- coding: utf-8 -*-
"""
security utils package.
"""

from pyrin.packaging.base import Package


class SecurityUtilsPackage(Package):
    """
    security utils package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'security.utils.component'
