# -*- coding: utf-8 -*-
"""
database bulk package.
"""

from pyrin.packaging.base import Package


class DatabaseBulkPackage(Package):
    """
    database bulk package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'database.bulk.component'
