# -*- coding: utf-8 -*-
"""
database session_factory package.
"""

from pyrin.packaging.base import Package


class DatabaseSessionFactoryPackage(Package):
    """
    database session factory package class.
    """

    NAME = __name__
    DEPENDS = []
