# -*- coding: utf-8 -*-
"""
cli structs module.
"""


class CLIGroups:
    """
    cli groups class.

    each cli handler group will be registered with its relevant group name.

    usage example:

    `python cli.py alembic upgrade --arg value`
    `python cli.py babel extract --arg value`
    `python cli.py template package`
    `python cli.py celery worker --arg value`
    `python cli.py security token --arg value`
    """
    pass
